@@ -43,7 +43,7 @@
 #include <lal/CoincInspiralEllipsoid.h>
 
 /**
- * \defgroup CoincInspiralEllipsoid_c Module CoincInspiralEllipsoid.c
+ * \defgroup CoincInspiralUtils_c Module CoincInspiralUtils.c
  * \ingroup CoincInspiralEllipsoid_h
  * \author Fairhurst, S.
  *
@@ -1122,7 +1122,7 @@ XLALRecreateCoincFromSngls(
       {
 	/* already have an event for this ifo */
 	XLALPrintError(
-		"Already have a single from this ifo with event id %lld",
+		"Already have a single from this ifo with event id %"LAL_UINT8_FORMAT,
 		eventId);
 	/* free memory */
 	while ( coincHead )
